from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class RevpaymentInvoiceConfig(AppConfig):
    name = 'revpayment.invoice'
    verbose_name = _('Invoice')
    label = 'revpayment.invoice'



from revpayment.utils import round
from revpayment.settings import api_settings


class BaseCalculations:
    tax_rate = api_settings.TAX_RATE

    def __init__(self, items, profile, config):
        self.items = items
        self.profile = profile
        self.config = config

    def get_sales(self):
        raise NotImplementedError

    def get_items(self):
        raise NotImplementedError

    def get_amount(self):
        raise NotImplementedError

    def get_invoice_items(self):
        raise NotImplementedError

    def get_calc_dict(self):
        fee, fee_tax = self.get_fee()
        sales, tax = self.get_sales()
        items_amt, items_tax = self.get_items()
        return self.format_result({
            'amount': sales + tax,
            'items_amount': items_amt,
            'items_tax': items_tax,
            'fee': fee,
            'fee_tax': fee_tax,
            'sales': sales,
            'tax': tax,
        })

    def format_result(self, data):
        for k, v in data.items():
            if type(v) is float:
                data[k] = int(v)
        return data

    @staticmethod
    def get_empty_dict():
        return {
            'amount': 0,
            'items_amount': 0,
            'items_tax': 0,
            'fee': 0,
            'fee_tax': 0,
            'sales': 0,
            'tax': 0,
        }


class Calculations(BaseCalculations):
    extra_invoices = [{'amount': 'fee', 'tax': 'fee_tax', 'name': '運費'}]

    def get_fee(self):
        return 0, 0

    def get_sales(self):
        fee, fee_tax = self.get_fee()
        items = self.items
        amount = sum([item['price'] for item in items])
        result = round(amount / (1 + self.tax_rate))
        tax = amount - result
        return result + fee, tax + fee_tax

    def get_items(self):
        items = self.items
        amount = sum([item['price'] for item in items])
        result = round(amount / (1 + self.tax_rate))
        tax = amount - result
        return result, tax

    def get_amount(self):
        sales, tax = self.get_sales()
        return sales + tax

    def get_invoice_items(self, category='B2C', calc={}):
        if category == 'B2C':
            items = [{
                'name': item['name'],
                'price': int(item['price'] / item['config']['quantity']),
                'amount': item['price'],
                'quantity': item['config']['quantity']
            } for item in self.items]
            for field in self.extra_invoices:
                value = calc.get(field['amount'], 0)
                tax = calc.get(field['tax'], 0)
                if value:
                    items.append({
                        'name': field['name'],
                        'price': round(value + tax),
                        'amount': round(value + tax),
                        'quantity': 1
                    })
            return items
        else:
            rate = 1 + self.tax_rate
            items = [{
                'name': item['name'],
                'price': round(item['price'] / (item['config']['quantity'] * rate)),
                'amount': round(item['price'] / rate),
                'quantity': item['config']['quantity']
            } for item in self.items]
            for field in self.extra_invoices:
                value = calc.get(field['amount'], 0)
                tax = calc.get(field['tax'], 0)
                if value:
                    items.append({
                        'name': field['name'],
                        'price': value,
                        'amount': value,
                        'quantity': 1
                    })
            return items



# Generated by Django 3.1.1 on 2020-10-22 09:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('revpayment', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='pay_deadline',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



from revpayment.settings import api_settings
from revpayment.logistics.models import Logistics
from revpayment.models import RedirectState
from datetime import datetime
from django.http import HttpResponse
import requests
import json

API_HOST = 'https://payment.revtel-api.com'
STG_HOST = 'https://payment-stg.revtel-api.com'
STATE = {
    'FAMI': {
        '300': 'processing',
        '310': 'uploading',
        '3024': 'transit',
        '3022': 'delivered',
        '3018': 'store_arrived'
    },
    'UNIMART': {
        '300': 'processing',
        '310': 'uploading',
        '2041': 'transit',
        '2067': 'delivered',
        '2063': 'store_arrived'
    },
    'UNIMARTC2C': {
        '300': 'processing',
        '310': 'uploading',
        '2068': 'transit',
        '2067': 'delivered',
        '2073': 'store_arrived'
    },
    'HILIFE': {
        '300': 'processing',
        '310': 'uploading',
        '3006': 'transit',
        '2067': 'delivered',
        '2063': 'store_arrived',
        '2073': 'store_arrived',
    },
    'ECAN': {
        '300': 'processing',
        '310': 'uploading',
        '3006': 'transit',
        '3003': 'delivered'
    },
    'TCAT': {
        '300': 'processing',
        '310': 'uploading',
        '3006': 'transit',
        '3003': 'delivered'
    }
}
STATE['FAMIC2C'] = STATE['FAMI']
STATE['HILIFEC2C'] = STATE['HILIFE']


class BaseLogistics:
    DEBUG = api_settings.DEBUG
    API_KEY = api_settings.CLIENT_SECRET
    VERSION = 'v2'
    logistic_provider = 'none'
    logistics_class = Logistics
    handler_class = api_settings.HANDLER_CLASS
    sender_name = api_settings.LOGISTIC_SENDER_NAME
    sender_phone = api_settings.LOGISTIC_SENDER_PHONE
    sender_cellphone = api_settings.LOGISTIC_SENDER_CELLPHONE
    callback_prefix = 'payment'

    def __init__(self, order):
        self.order = order

    @property
    def state(self):
        return RedirectState.objects.get(order_id=self.order.id)

    @property
    def cart(self):
        return json.loads(self.order.cart)

    @property
    def logistic_type(self):
        config = self.cart['config']
        return config['logistic_type']

    @property
    def logistic_subtype(self):
        config = self.cart['config']
        return config['logistic_subtype']

    @property
    def url(self):
        if self.DEBUG:
            return f'{STG_HOST}/{self.VERSION}'
        return f'{API_HOST}/{self.VERSION}'

    def _post(self, url, data):
        return requests.post(url, json=data, headers={'x-api-key': self.API_KEY, 'Content-Type': 'application/json'})

    def map_logistic_callback(self, data):
        raise NotImplementedError

    def build_logistic_data(self):
        raise NotImplementedError

    def on_logistic_result(self, data):
        mapped = self.map_logistic_callback(data)
        if not mapped:
            return
        fields = [f.name for f in self.logistics_class._meta.get_fields()]

        try:
            logistics = self.logistics_class.objects.get(
                order=self.order
            )
        except:
            logistics = self.logistics_class.objects.create(
                order=self.order,
                logistic_provider=self.logistic_provider,
                logistic_type=self.logistic_type,
                logistic_subtype=self.logistic_subtype
            )
        for k, v in mapped.items():
            if k in fields:
                setattr(logistics, k, v)
        try:
            detail = json.loads(logistics.detail)
        except:
            detail = []
        detail.append(data)
        detail_str = json.dumps(detail, ensure_ascii=False)
        logistics.detail = detail_str
        logistics.save()

        changed = {'_payment_status': self.order.payment_status}
        if self.order.payment_subtype == 'cvs_cod' and mapped.get('logistic_status') == 'delivered':
            self.order.payment_status = 'success'
            self.order.save()
            changed['payment_status'] = self.order.payment_status
            handler = self.handler_class(order=self.order)
            handler.on_payment_result(changed)
        return logistics

    def create_logistics(self):
        data = self.build_logistic_data()
        url = f'{self.url}/{self.logistic_provider}/logistics/create'

        resp = self._post(url, data=data)
        resp_json = resp.json()
        resp.raise_for_status()
        logistics = self.on_logistic_result(resp_json)
        return logistics

    def callback(self, data):
        self.on_logistic_result(data)

    def build_cvs_data(self, **kwargs):
        raise NotImplementedError

    def cvs_map(self, logistic_subtype, collection, state):
        url = f'{self.url}/{self.logistic_provider}/cvs/map'
        data = self.build_cvs_data(logistic_subtype, collection, state)
        resp = self._post(url, data=data)
        resp.raise_for_status()
        resp_json = resp.json()
        return HttpResponse(resp_json['html'], charset='utf-8')


class EcpayLogistics(BaseLogistics):
    logistic_provider = 'ecpay'

    def build_logistic_data(self):
        config = self.cart['config']
        extra = {}
        if self.order.payment_subtype == 'cvs_cod':
            extra['IsCollection'] = 'Y'

        if self.logistic_type == 'cvs':
            extra['LogisticsType'] = 'CVS'
            extra['LogisticsSubType'] = config['logistic_subtype']
            extra['ReceiverStoreID'] = config['CVSStoreID']
            extra['LogisticsC2CReplyURL'] = f'{api_settings.WEB_HOST}/order?id={self.order.id}'

        if self.logistic_type == 'home':
            extra['LogisticsType'] = 'HOME'
            extra['LogisticsSubType'] = config.get('logistic_subtype', 'TCAT')
            extra['ReceiverAddress'] = config['ReceiverAddress']
            extra['ReceiverZipCode'] = config['zip_code']
            extra['Temperature'] = '0001'
            extra['Distance'] = '00'
            extra['Specification'] = '0001'

        return {
            'MerchantTradeDate': datetime.now().strftime('%Y/%m/%d %H:%M:%S'),
            'MerchantTradeNo': self.order.id,
            'SenderName': self.sender_name,
            'SenderPhone': self.sender_phone,
            'SenderCellPhone': self.sender_cellphone,
            'ReceiverName': config['name'],
            'ReceiverEmail': config['email'],
            'ReceiverPhone': config['phone'],
            'ReceiverCellPhone': config['phone'],
            'GoodsAmount': self.order.amount,
            'GoodsName': self.order.title,
            'ServerReplyURL': f'{api_settings.API_HOST}/{self.callback_prefix}/logistics/callback?state={self.state.state}',
            **extra
        }

    def map_logistic_callback(self, data):
        code = data['RtnCode']
        try:
            status = STATE[self.logistic_subtype][code]
        except:
            status = 'exception'
        return {
            'logistic_status': status,
        }

    def build_cvs_data(self, logistic_subtype, collection, state, is_mobile=False):
        extra = {'IsCollection': 'N'}
        if collection:
            extra['IsCollection'] = 'Y'
        return {
            'LogisticsType': 'CVS',
            'LogisticsSubType': logistic_subtype,
            'ServerReplyURL': f'{api_settings.API_HOST}/{self.callback_prefix}/logistics/cvs/callback?state={state.state}',
            'Device': 1 if is_mobile else 0,
            **extra
        }



import json
from rest_framework import exceptions
from revpayment.settings import api_settings


class Cart:
    tax_rate = api_settings.TAX_RATE
    item_class = api_settings.ITEM_CLASS
    calc_class = api_settings.CALC_CLASS
    config_class = api_settings.CONFIG_CLASS

    def __init__(self, profile):
        self.profile = profile
        self.payment = profile.payment
        self.cart = json.loads(self.payment.cart)

    def _save_cart(self):
        config = self.cart['config']
        calc = self.calc_class(
            items=self.cart['items'], profile=self.profile, config=config)
        self.cart['calculations'] = calc.get_calc_dict()
        self.payment.cart = json.dumps(self.cart, ensure_ascii=False)
        self.payment.save()

    def add_item(self, item):
        logic = self.item_class(item=item, profile=self.profile)
        verified_item = logic.get_item()
        self.cart['items'].append(verified_item)
        self._save_cart()

    def delete_item(self, index):
        if index > len(self.cart['items']) - 1:
            raise exceptions.ValidationError
        items = self.cart['items']
        del items[index]
        self.cart['items'] = items
        self._save_cart()

    def edit_item(self, index, config):
        items = self.cart['items']
        if index > len(self.cart['items']) - 1:
            raise exceptions.ValidationError
        item = items[index]
        item = {**item, 'config': {**item['config'],
                                   **config}, 'product': item['product']['id']}
        item = self.item_class(item=item, profile=self.profile)
        self.cart['items'][index] = item.get_item()
        self._save_cart()

    def edit_config(self, update_config):
        config = self.config_class(config=self.cart['config'])
        config_dict = config.update_config(**update_config)
        self.cart['config'] = config_dict
        self._save_cart()

    def clear_cart(self):
        self.cart = {
            **self.cart,
            'items': [],
            'calculations': self.calc_class.get_empty_dict()
        }
        self._save_cart()

    def _build_order_title(self):
        return 'REVTEL-ORDER'

    def get_order_data(self):
        return {
            'amount': self.cart['calculations']['amount'],
            'cart': json.dumps(self.cart, ensure_ascii=False),
            'title': self._build_order_title(),
            'receiver_name': self.cart['config'].get('name', self.profile.name),
            'receiver_email': self.cart['config'].get('email', self.profile.email),
            'receiver_phone': self.cart['config'].get('phone', self.profile.phone),
            'receiver_address': self.cart['config'].get('address', self.profile.address),
        }

    def on_order_will_create(self, data):
        return self.get_order_data()

    @staticmethod
    def get_empty_cart(dumps=False, **kwargs):
        data = {
            'config': {**kwargs},
            'items': [],
            'calculations': Cart.calc_class.get_empty_dict()
        }
        if dumps:
            return json.dumps(data)
        return data



from django.db import models
from django.contrib import admin
from revpayment.settings import api_settings
from django.conf import settings


class Invoice(models.Model):
    INVOICE_TYPES = (
        ('neweb', 'neweb'),
        ('ecpay', 'ecpay')
    )
    INVOICE_STATUS = (
        ('success', 'success'),
        ('failure', 'failure'),
        ('void', 'void')
    )

    CATEGORY = (
        ('B2B', 'B2B'),
        ('B2C', 'B2C')
    )

    CARRIER_TYPE = (
        ('no_carrier', 'no_carrier'),
        ('mobile', 'mobile'),
        ('npc', 'npc'),
        ('donation', 'donation'),
        ('default', 'default')
    )

    category = models.CharField(max_length=10, choices=CATEGORY, default='B2C')
    order = models.ForeignKey(api_settings.ORDER_CLASS, null=True, on_delete=models.CASCADE)
    amount = models.IntegerField(default=0)
    invoice_type = models.CharField(max_length=64, choices=INVOICE_TYPES, default='neweb')
    invoice_status = models.CharField(max_length=64, choices=INVOICE_STATUS)
    invoice_number = models.CharField(max_length=128, null=True)
    carrier_type = models.CharField(max_length=128, choices=CARRIER_TYPE, default='no_carrier', null=True)
    carrier_number = models.CharField(max_length=128, blank=True, null=True)
    uni_no = models.CharField(max_length=128, blank=True, null=True)
    print_flag = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True)
    detail = models.TextField(blank=True)

    class Meta:
        abstract = 'revpayment.invoice' not in settings.INSTALLED_APPS


class InvoiceAdmin(admin.ModelAdmin):
    list_display = ('id', 'order', 'invoice_type', 'created')


if not Invoice._meta.abstract:
    admin.site.register(Invoice, InvoiceAdmin)



from revpayment import TAX_RATE
from rest_framework import exceptions
from revpayment.settings import api_settings


class BaseItem:
    tax_rate = TAX_RATE
    product_class = api_settings.PRODUCT_CLASS
    serializer_class = api_settings.PRODUCT_SERIALIZER_CLASS

    def __init__(self, item, profile):
        self.item = item
        self.profile = profile
        try:
            self.product = self.product_class.objects.get(id=item['product'])
        except:
            raise exceptions.NotFound
        self.validate_config()

    def _get_serialized_product(self):
        return self.serializer_class(self.product).data

    def get_item(self):
        raise NotImplementedError

    def validate_config(self):
        raise NotImplementedError

    def get_price(self):
        raise NotImplementedError

    @classmethod
    def get_credit_item(cls, credit, naming_format='{} 點'):
        return {
            'name': naming_format.format(credit),
            'price': credit,
            'config': {'quantity': 1}
        }


class Item(BaseItem):
    def validate_config(self):
        if 'config' not in self.item:
            raise exceptions.ValidationError

        config = self.item['config']
        if 'quantity' not in config:
            raise exceptions.ValidationError

    def get_item(self):
        extra = {}
        price = self.get_price()
        return {
            'product': self._get_serialized_product(),
            'price': price,
            'config': self.item['config'],
            'name': self.product.name,
            **extra
        }

    def get_price(self):
        amount = self.product.price * self.item['config']['quantity']
        return amount



from revpayment.settings import api_settings
from django.shortcuts import redirect

default_app_config = 'revpayment.logistics.apps.LogisticsConfig'

class LogisticsSDK:
    def __init__(self, order=None, logistic_provider='ecpay'):
        self.order = order
        self.logistic_provider = logistic_provider

    @property
    def provider_class(self):
        from revpayment.logistics.providers import EcpayLogistics
        if self.logistic_provider == 'ecpay':
            return EcpayLogistics

    @property
    def provider(self):
        return self.provider_class(order=self.order)

    def create_logistics(self, **kwargs):
        return self.provider.create_logistics(**kwargs)

    def cvs_map(self, logistic_subtype, state, collection=False):
        return self.provider.cvs_map(logistic_subtype, collection, state)

    def cvs_callback(self, result, profile):
        from revpayment.carts import Cart
        cart = Cart(profile)
        cart.edit_config(result)
        return redirect(f'{api_settings.WEB_HOST}/cart')



default_app_config = 'revpayment.invoice.apps.RevpaymentInvoiceConfig'


class InvoiceSDK:
    def __init__(self, order=None, invoice_type='neweb'):
        self.order = order
        self.invoice_type = invoice_type

    def get_provider_class(self):
        from .providers import EcpayProvider, NewebProvider
        if self.invoice_type == 'ecpay':
            return EcpayProvider
        elif self.invoice_type == 'neweb':
            return NewebProvider

    def get_provider(self, **kwargs):
        provider_class = self.get_provider_class()
        return provider_class(order=self.order, **kwargs)

    def issue_invoice(self, **kwargs):
        invoice = self.get_provider(**kwargs)
        resp = invoice.issue_invoice()
        return resp



# Generated by Django 3.1.1 on 2020-09-16 07:51

from django.db import migrations, models
import django.db.models.deletion
from revpayment.settings import api_settings


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(api_settings.ORDER_CLASS._meta.label),
    ]

    operations = [
        migrations.CreateModel(
            name='Logistics',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('logistic_provider', models.CharField(choices=[('ecpay', 'ecpay')], default='ecpay', max_length=128)),
                ('logistic_type', models.CharField(choices=[('home', 'home'), ('cvs', 'cvs')], default='cvs', max_length=128)),
                ('logistic_subtype', models.CharField(choices=[('FAMI', 'FAMI'), ('FAMIC2C', 'FAMIC2C'), ('UNIMART', 'UNIMART'), ('UNIMARTC2C', 'UNIMARTC2C'), ('HILIFE', 'HIFIFE'), ('HILIFEC2C', 'HIFIFEC2C'), ('ECAN', 'ECAN'), ('TCAT', 'TCAT')], default='UNIMARTC2C', max_length=128)),
                ('logistic_status', models.CharField(choices=[('processing', 'processing'), ('uploading', 'uploading'), ('transit', 'transit'), ('delivered', 'delivered'), ('store_arrived', 'store_arrived')], default='processing', max_length=128)),
                ('detail', models.TextField(blank=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=api_settings.ORDER_CLASS._meta.label)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.1.1 on 2020-09-16 07:49

from django.db import migrations, models
import django.db.models.deletion
import revpayment.models
from revpayment.settings import api_settings
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(api_settings.BUYER_CLASS._meta.label),
    ]

    operations = [
        migrations.CreateModel(
            name='RedirectState',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('state', models.UUIDField(default=uuid.uuid4)),
                ('redirect_url', models.URLField(default='http://localhost:8000')),
                ('payment_type', models.CharField(choices=[('offline', 'offline'), ('ecpay', 'ecpay'), ('neweb', 'neweb')], default='neweb', max_length=20)),
                ('payment_subtype', models.CharField(choices=[('default', 'default'), ('credit', 'credit'), ('cvs_cod', 'cvs_cod'), ('cvs', 'cvs'), ('atm', 'atm'), ('web_atm', 'web_atm'), ('barcode', 'barcode')], default='default', max_length=20)),
                ('cart', models.TextField()),
                ('order_id', models.CharField(default=revpayment.models.gen_order_id, max_length=128)),
                ('order_type', models.CharField(default='default', max_length=128)),
                ('buyer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=api_settings.BUYER_CLASS._meta.label)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PaymentInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('cart', models.TextField(blank=True, default=revpayment.models.empty_cart)),
                ('credit', models.IntegerField(default=0)),
                ('trading_slang', models.CharField(default=revpayment.models.random_digits, max_length=20)),
                ('owner', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='payment', to=api_settings.BUYER_CLASS._meta.label)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.CharField(default=revpayment.models.gen_order_id, max_length=128, primary_key=True, serialize=False)),
                ('order_type', models.CharField(choices=[('default', 'default'), ('credit', 'credit')], default='default', max_length=128)),
                ('amount', models.FloatField(default=0)),
                ('title', models.CharField(default='order', max_length=100)),
                ('description', models.CharField(default='no description', max_length=256)),
                ('currency', models.CharField(choices=[('TWD', 'TWD'), ('USD', 'USD'), ('JPY', 'JPY')], default='TWD', max_length=8)),
                ('payment_type', models.CharField(choices=[('offline', 'offline'), ('ecpay', 'ecpay'), ('neweb', 'neweb')], default='neweb', max_length=20)),
                ('payment_subtype', models.CharField(choices=[('default', 'default'), ('credit', 'credit'), ('cvs_cod', 'cvs_cod'), ('cvs', 'cvs'), ('atm', 'atm'), ('web_atm', 'web_atm'), ('barcode', 'barcode')], default='default', max_length=20)),
                ('payment_status', models.CharField(choices=[('success', 'success'), ('failure', 'failure'), ('refunded', 'refunded'), ('logistics_created', 'logistics_created'), ('form_created', 'form_created'), ('code_generated', 'code_generated')], default='waiting', max_length=20)),
                ('data', models.TextField(blank=True, null=True)),
                ('cart', models.TextField(blank=True, null=True)),
                ('receiver_email', models.EmailField(max_length=254)),
                ('receiver_name', models.CharField(blank=True, max_length=20)),
                ('receiver_phone', models.CharField(blank=True, max_length=20)),
                ('receiver_address', models.CharField(blank=True, max_length=128)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('payment_transaction_detail', models.TextField(blank=True)),
                ('payment_redirect_detail', models.TextField(blank=True)),
                ('code_no', models.CharField(blank=True, max_length=128)),
                ('bank_code', models.CharField(blank=True, max_length=128)),
                ('bank_account', models.CharField(blank=True, max_length=128)),
                ('barcode_1', models.CharField(blank=True, max_length=128)),
                ('barcode_2', models.CharField(blank=True, max_length=128)),
                ('barcode_3', models.CharField(blank=True, max_length=128)),
                ('buyer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=api_settings.BUYER_CLASS._meta.label)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.1.1 on 2020-09-16 07:51

from django.db import migrations, models
import django.db.models.deletion
from revpayment.settings import api_settings


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(api_settings.ORDER_CLASS._meta.label),
    ]

    operations = [
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('category', models.CharField(choices=[('B2B', 'B2B'), ('B2C', 'B2C')], default='B2C', max_length=10)),
                ('amount', models.IntegerField(default=0)),
                ('invoice_type', models.CharField(choices=[('neweb', 'neweb'), ('ecpay', 'ecpay')], default='neweb', max_length=64)),
                ('invoice_status', models.CharField(choices=[('success', 'success'), ('failure', 'failure'), ('void', 'void')], max_length=64)),
                ('invoice_number', models.CharField(max_length=128, null=True)),
                ('carrier_type', models.CharField(choices=[('no_carrier', 'no_carrier'), ('mobile', 'mobile'), ('npc', 'npc'), ('donation', 'donation'), ('default', 'default')], default='no_carrier', max_length=128, null=True)),
                ('carrier_number', models.CharField(blank=True, max_length=128, null=True)),
                ('uni_no', models.CharField(blank=True, max_length=128, null=True)),
                ('print_flag', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('detail', models.TextField(blank=True)),
                ('order', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=api_settings.ORDER_CLASS._meta.label)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.db import models
from django.conf import settings
from django.contrib import admin
from revpayment.settings import api_settings

# Create your models here.


class Logistics(models.Model):
    PROVIDERS = (
        ('ecpay', 'ecpay'),
    )
    TYPES = (
        ('home', 'home'),
        ('cvs', 'cvs'),
    )
    SUBTYPES = (
        ('FAMI', 'FAMI'),
        ('FAMIC2C', 'FAMIC2C'),
        ('UNIMART', 'UNIMART'),
        ('UNIMARTC2C', 'UNIMARTC2C'),
        ('HILIFE', 'HIFIFE'),
        ('HILIFEC2C', 'HIFIFEC2C'),
        ('ECAN', 'ECAN'),
        ('TCAT', 'TCAT'),
    )
    STATUS = (
        ('processing', 'processing'),
        ('uploading', 'uploading'),
        ('transit', 'transit'),
        ('delivered', 'delivered'),
        ('store_arrived', 'store_arrived'),
    )
    order = models.ForeignKey(api_settings.ORDER_CLASS, on_delete=models.CASCADE)
    logistic_provider = models.CharField(max_length=128, default='ecpay', choices=PROVIDERS)
    logistic_type = models.CharField(max_length=128, default='cvs', choices=TYPES)
    logistic_subtype = models.CharField(max_length=128, default='UNIMARTC2C', choices=SUBTYPES)
    logistic_status = models.CharField(max_length=128, default='processing', choices=STATUS)
    detail = models.TextField(blank=True)
    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = 'revpayment.logistics' not in settings.INSTALLED_APPS


if not Logistics._meta.abstract:
    admin.site.register(Logistics)



from django.conf import settings
from django.utils.module_loading import import_string


DEFAULTS = {
    # Base API policies
    'DEBUG': True,
    'PAYMENT_VERSION': 'v3',
    'TAX_RATE': 0.05,
    'LOGISTIC_SENDER_NAME': '',
    'LOGISTIC_SENDER_PHONE': '',
    'LOGISTIC_SENDER_CELLPHONE': '',
    'ORDER_CLASS': 'revpayment.models.Order',
    'ITEM_CLASS': 'revpayment.items.Item',
    'CALC_CLASS': 'revpayment.calculations.Calculations',
    'CONFIG_CLASS': 'revpayment.config.Config',
    'HANDLER_CLASS': 'revpayment.handlers.Handler',
    'INVOICE_CLASS': 'revpayment.invoice.models.Invoice',
    'DEFAULT_SDK_CLASS': 'revpayment.checkout.CheckoutSDK',
    'DEFAULT_CART_CLASS': 'revpayment.carts.Cart',
    'DEFAULT_NEWEB_CLASS': 'revpayment.payments.NewebPayment',
    'DEFAULT_ECPAY_CLASS': 'revpayment.payments.EcpayPayment',
    'DEFAULT_CREDIT_CLASS': 'revpayment.payments.CreditPayment',
}


# List of settings that may be in string import notation.
IMPORT_STRINGS = [
    'ORDER_CLASS', 'ITEM_CLASS', 'CALC_CLASS', 'CONFIG_CLASS', 'HANDLER_CLASS',
    'INVOICE_CLASS', 'BUYER_CLASS', 'PRODUCT_CLASS', 'PRODUCT_SERIALIZER_CLASS',
    'VERIFY_METHOD', 'DEFAULT_SDK_CLASS', 'DEFAULT_CART_CLASS', 'DEFAULT_NEWEB_CLASS',
    'DEFAULT_ECPAY_CLASS', 'DEFAULT_CREDIT_CLASS',
]


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if val is None:
        return None
    elif isinstance(val, str):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        return import_string(val)
    except ImportError as e:
        msg = "Could not import '%s' for API setting '%s'. %s: %s." % (val, setting_name, e.__class__.__name__, e)
        raise ImportError(msg)


class APISettings:
    def __init__(self, defaults=None, import_strings=None):
        self.defaults = defaults or DEFAULTS
        self.import_strings = import_strings or IMPORT_STRINGS

    @property
    def user_settings(self):
        return getattr(settings, 'REVPAYMENT', {})

    def __getattr__(self, attr):
        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        setattr(self, attr, val)
        return val


api_settings = APISettings(DEFAULTS, IMPORT_STRINGS)



# Generated by Django 3.1.1 on 2020-11-04 07:22

from django.db import migrations, models
import revpayment.models


class Migration(migrations.Migration):

    dependencies = [
        ('revpayment', '0002_order_pay_deadline'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentinfo',
            name='uid',
            field=models.CharField(default=revpayment.models.uuid_hex, max_length=30),
        ),
    ]



import requests
import json
from django.utils import timezone
from django.shortcuts import redirect
from django.http import HttpResponse
from datetime import datetime
from revpayment.settings import api_settings
from revpayment.logistics import LogisticsSDK


class BasePayment:
    URL = f'https://payment.revtel-api.com/{api_settings.PAYMENT_VERSION}'
    STAGE_URL = f'https://payment-stg.revtel-api.com/{api_settings.PAYMENT_VERSION}'
    PATH = None
    DEBUG = api_settings.DEBUG
    API_KEY = api_settings.CLIENT_SECRET
    WEB_HOST = api_settings.WEB_HOST
    API_HOST = api_settings.API_HOST
    order_class = api_settings.ORDER_CLASS
    handler_class = api_settings.HANDLER_CLASS
    payment_type = 'base'

    def __init__(self, cart, buyer, order_id, payment_subtype='default'):
        self.cart = cart
        self.payment_subtype = payment_subtype
        self.buyer = buyer
        self.order_id = order_id

    @property
    def host(self):
        if self.DEBUG:
            return self.STAGE_URL
        return self.URL

    def gen_order_id(self):
        return self.order_id

    def get_url(self):
        url = self.host
        return url + self.PATH

    def build_data(self, data):
        raise NotImplementedError

    def cvs_cod_checkout(self, state):
        cart = json.loads(state.cart)
        items = cart['items']
        config = cart['config']
        calc = cart['calculations']
        order = self.order_class.objects.create(cart=state.cart,
                                                id=state.order_id,
                                                title=items[0]['name'],
                                                receiver_email=config['email'],
                                                receiver_name=config['name'],
                                                receiver_phone=config.get('email', ''),
                                                receiver_address=config.get('address', ''),
                                                payment_type=state.payment_type,
                                                payment_subtype=state.payment_subtype,
                                                buyer=self.buyer,
                                                amount=calc['amount'],
                                                )
        changed = {'__created': True}
        hander = self.handler_class(order=order)
        hander.on_payment_result(changed)
        sdk = LogisticsSDK(order=order)
        sdk.create_logistics()
        return order

    def checkout(self, state):
        if self.payment_subtype == 'cvs_cod':
            order = self.cvs_cod_checkout(state)
            return redirect(f'{self.WEB_HOST}/order?id={order.id}')
        data = self.build_data(state)
        resp = requests.post(url=self.get_url(), json=data, headers={
            'Content-Type': 'application/json',
            'x-api-key': self.API_KEY
        })
        resp.raise_for_status()
        resp_json = resp.json()
        return redirect(resp_json['url'])

    def map_callback_data(self, data):
        raise NotImplementedError

    def callback(self, state, data):
        config = json.loads(state.cart)['config']
        items = json.loads(state.cart)['items']
        mapped = self.map_callback_data(data)
        changed = {'__created': False}
        if 'id' not in mapped:
            raise ValueError('id is required in mapped data')
        try:
            order = self.order_class.objects.get(
                id=mapped['id']
            )
            del mapped['id']
            for k, v in mapped.items():
                if getattr(order, k) != v:
                    changed[k] = v
                    changed['_' + k] = getattr(order, k)
                setattr(order, k, v)
            order.save()
        except:
            order = self.order_class.objects.create(
                **mapped,
                cart=state.cart,
                title=items[0]['name'],
                receiver_email=config['email'],
                receiver_name=config['name'],
                receiver_phone=config.get('email', ''),
                receiver_address=config.get('address', ''),
                payment_type=state.payment_type,
                payment_subtype=state.payment_subtype,
                payment_transaction_detail=json.dumps(
                    data, ensure_ascii=False),
                buyer=self.buyer,
                order_type=state.order_type
            )
            changed = mapped
            changed['__created'] = True

        state.order_id = order.id
        state.save()
        return order, changed

    def map_redirect_data(self, data):
        raise NotImplementedError

    def customer_redirect(self, state, data):
        config = json.loads(state.cart)['config']
        items = json.loads(state.cart)['items']
        mapped = self.map_redirect_data(data)
        changed = {'__created': False}
        if 'id' not in mapped:
            if not state.order_id:
                raise ValueError('state order_id is null')
            mapped['id'] = state.order_id
        try:
            order = self.order_class.objects.get(
                id=mapped['id']
            )
            del mapped['id']
            for k, v in mapped.items():
                if getattr(order, k) != v:
                    changed[k] = v
                    changed['_' + k] = getattr(order, k)
                setattr(order, k, v)
            order.save()
        except:
            order = self.order_class.objects.create(
                **mapped,
                payment_type=self.payment_type,
                payment_redirect_detail=json.dumps(data, ensure_ascii=False),
                cart=state.cart,
                title=items[0]['name'],
                receiver_email=config['email'],
                receiver_name=config['name'],
                receiver_phone=config.get('email', ''),
                receiver_address=config.get('address', ''),
                buyer=self.buyer
            )
            changed = mapped
            changed['__created'] = True

        state.order_id = order.id
        state.save()
        return order, mapped


class NewebPayment(BasePayment):
    PATH = '/newebpay/request'
    payment_type = 'neweb'
    PAYMENT_TABLE = {
        'default': {
            'CREDIT': 1,
            'CVSCOM': 0,
            'BARCODE': 1,
            'CVS': 1,
            'WEBATM': 1,
            'VACC': 1,
        },
        'credit': {
            'CREDIT': 1,
            'CVSCOM': 0,
            'BARCODE': 0,
            'CVS': 0,
            'WEBATM': 0,
        },
        'cvs_cod': {
            'CREDIT': 0,
            'CVSCOM': 2,
            'BARCODE': 0,
            'CVS': 0,
            'WEBATM': 0,
        },
        'web_atm': {
            'CREDIT': 0,
            'CVSCOM': 0,
            'BARCODE': 0,
            'CVS': 0,
            'WEBATM': 1,
        },
        'cvs': {
            'CREDIT': 0,
            'CVSCOM': 0,
            'BARCODE': 0,
            'CVS': 1,
            'WEBATM': 0,
        },
        'atm': {
            'VACC': 1,
            'CREDIT': 0,
            'CVSCOM': 0,
            'BARCODE': 0,
            'CVS': 0,
            'WEBATM': 0,
        }
    }

    def build_data(self, state):
        payment_subtype = self.PAYMENT_TABLE[self.payment_subtype]
        cart = self.cart
        items = cart['items']
        now = timezone.now()
        data = {
            'TimeStamp': str(int(now.timestamp())),
            'Amt': int(cart['calculations']['amount']),
            'Email': cart['config']['email'],
            'ItemDesc': items[0]['name'],
            'MerchantOrderNo': self.gen_order_id(),
            'CREDIT': 1,
            'CVSCOM': cart['config'].get('CVSCOM', 0),
            'BARCODE': 1,
            'CVS': 1,
            'WEBATM': 1,
            **payment_subtype
        }
        return data

    def map_callback_data(self, data):
        return_code = data.get('Status')
        payment_data = {
            'id': data['Result']['MerchantOrderNo'],
            'payment_status': 'success' if return_code == 'SUCCESS' else 'failure',
            'amount': data['Result']['Amt']
        }
        return payment_data

    def map_redirect_data(self, data):
        payload = {}
        if not data.get('Result'):
            return {}
        result = data['Result']
        payment_type = result['PaymentType']
        if data['Status'] == 'SUCCESS':
            payload['id'] = self.order_id
            payload['amount'] = result['Amt'] if type(
                result['Amt']) in [float, int] else int(result['Amt'])
            payload['payment_status'] = 'code_generated'
            payload['pay_deadline'] = result['ExpireDate'] + 'T' + result['ExpireTime']
            if payment_type == 'CVS':
                payload['payment_subtype'] = 'cvs'
                payload['code_no'] = result['CodeNo']
            elif payment_type == 'BARCODE':
                payload['payment_subtype'] = 'barcode'
                payload['barcode_1'] = result['Barcode_1']
                payload['barcode_2'] = result['Barcode_2']
                payload['barcode_3'] = result['Barcode_3']
            elif payment_type == 'VACC':
                payload['payment_subtype'] = 'atm'
                payload['bank_code'] = result['BankCode']
                payload['bank_account'] = result['CodeNo']

        return payload


class EcpayPayment(BasePayment):
    PATH = '/ecpay/request'
    payment_type = 'ecpay'
    PAYMENT_TABLE = {
        'default': {'ChoosePayment': 'ALL'},
        'credit': {'ChoosePayment': 'Credit'},
        'web_atm': {'ChoosePayment': 'WebATM'},
        'atm': {'ChoosePayment': 'ATM'},
        'cvs': {'ChoosePayment': 'CVS'},
        'barcode': {'ChoosePayment': 'BARCODE'},
    }

    def build_data(self, state):
        now = timezone.now()
        payment_subtype = self.PAYMENT_TABLE[self.payment_subtype]
        datetime_str = now.strftime('%Y/%m/%d %H:%M:%S')
        trade_no = self.gen_order_id()
        cart = self.cart
        calc = cart['calculations']
        config = cart['config']
        items = cart['items']
        data = {
            'MerchantTradeNo': trade_no,
            'MerchantTradeDate': datetime_str,
            'TotalAmount': int(calc['amount']),
            'ItemName': '#'.join([item['name'] for item in items]),
            'TradeDesc': 'None',
            'ChoosePayment': 'ALL',
            'InvoiceMark': 'N',
            'BindingCard': 1,
            'MerchantMemberID': self.buyer.payment.uid,
            **payment_subtype
        }

        return {**data}

    def get_payment_type(self, payment_type):
        subtype = 'default'
        if payment_type.startswith('CVS_'):
            subtype = 'cvs'
        elif payment_type.startswith('BARCODE_'):
            subtype = 'barcode'
        elif payment_type.startswith('ATM_'):
            subtype = 'atm'
        elif payment_type.startswith('Credit_'):
            subtype = 'credit'
        return subtype

    def map_callback_data(self, data):
        return_code = data.get('RtnCode')
        cart = self.cart
        payment_data = {
            'amount': cart['calculations']['amount'],
            'payment_status': 'success' if return_code == '1' else 'failure',
            'id': data['MerchantTradeNo'],
        }
        return payment_data

    def map_redirect_data(self, data):
        payment_type = data['PaymentType']
        code = data['RtnCode']
        payload = {
            'amount': data['TradeAmt']
        }
        if payment_type.startswith('CVS_') and code == '10100073':
            payload['payment_status'] = 'code_generated'
            payload['payment_subtype'] = 'cvs'
            payload['code_no'] = data['PaymentNo']
            payload['pay_deadline'] = data['ExpireDate']
        elif payment_type.startswith('BARCODE_') and code == '10100073':
            payload['payment_status'] = 'code_generated'
            payload['payment_subtype'] = 'barcode'
            payload['barcode_1'] = data['Barcode1']
            payload['barcode_2'] = data['Barcode2']
            payload['barcode_3'] = data['Barcode3']
            payload['pay_deadline'] = data['ExpireDate']
        elif payment_type.startswith('ATM_') and code == '2':
            payload['payment_status'] = 'code_generated'
            payload['payment_subtype'] = 'atm'
            payload['bank_code'] = data['BankCode']
            payload['bank_account'] = data['vAccount']
            payload['pay_deadline'] = datetime.strptime(data['ExpireDate'], '%Y/%m/%d')
        return payload


class CreditPayment(BasePayment):
    payment_type = 'credit'

    def map_callback_data(self, data):
        return {
            'id': self.gen_order_id(),
            **data
        }

    def checkout(self, state):
        buyer = self.buyer
        cart = self.cart
        calc = cart['calculations']
        if buyer.credit < calc['amount']:
            raise

        buyer.credit -= calc['amount']
        buyer.save()

        payload = {
            'amount': calc['amount'],
            'payment_status': 'success',
        }
        order, changed = self.callback(state, payload)
        return redirect(f'{self.WEB_HOST}/order?id={order.id}')

    def static_payment_result_page(self, order):
        return HttpResponse('<html> \
                        <head> \
                            <title>PaymentResult</title> \
                        </head> \
                        <body> \
                            <div> \
                                <h2>PaymentStatus</h2>\
                                <h3>{}</h3>\
                            </div> \
                        </body>\
                    </html>'.format(order.payment_status),
                            content_type='text/html')



from revpayment.settings import api_settings
import requests
from django.utils import timezone
from revpayment.utils import round
from revpayment.calculations import Calculations
import json
from revpayment.invoice.exceptions import MapResultError
from revpayment.models import gen_order_id


class BaseProvider:
    TAX_RATE = api_settings.TAX_RATE
    DEBUG = api_settings.DEBUG
    API_KEY = api_settings.CLIENT_SECRET
    invoice_class = api_settings.INVOICE_CLASS
    calc_class = api_settings.CALC_CLASS
    VERSION = 'v2'
    invoice_type = None
    _category = None
    _carrier_type = None
    _carrier_num = None
    _uni_no = None
    _receiver_email = None
    _slang = None

    def __init__(self, order=None, category=None, carrier_type=None, carrier_num=None, uni_no=None, receiver_email=None, slang=None, company_title=None, cart=None):
        self.order = order
        self._category = category
        self._carrier_type = carrier_type
        self._carrier_num = carrier_num
        self._uni_no = uni_no
        self._company_title = company_title
        self._receiver_email = receiver_email
        self._slang = slang
        self._cart = cart

    @property
    def invoice_id(self):
        if self.created:
            return gen_order_id()
        return self.order.id

    @property
    def created(self):
        if self.invoice_class.objects.filter(order=self.order):
            return True
        return False

    @property
    def receiver_email(self):
        if self._receiver_email:
            return self._receiver_email
        if self.order:
            return self.order.receiver_email
        return self.cart['config']['email']

    @property
    def slang(self):
        if self.order:
            return self.order.buyer.payment.trading_slang
        return self._slang

    @property
    def category(self):
        return self._category if self._category else self.cart['config'].get('category', None)

    @property
    def company_title(self):
        return self._company_title if self._company_title else self.cart['config'].get('company_title', None)

    @property
    def carrier_type(self):
        return self._carrier_type if self._carrier_type else self.cart['config'].get('carrier_type', None)

    @property
    def carrier_number(self):
        return self._carrier_num if self._carrier_num else self.cart['config'].get('carrier_num', None)

    @property
    def uni_no(self):
        return self._uni_no if self._uni_no else self.cart['config'].get('uni_no', None)

    @property
    def cart(self):
        if self._cart:
            return json.loads(self._cart)
        if self.order:
            return json.loads(self.order.cart)
        return None

    @property
    def items(self):
        if self.order:
            cart = self.cart
            calc = self.calc_class(items=cart['items'], config=cart['config'], profile=self.order.buyer)
            return calc.get_invoice_items(category=self.category, calc=cart['calculations'])
        return None

    @property
    def host(self):
        if self.DEBUG:
            return f'https://payment-stg.revtel-api.com/{self.VERSION}'
        else:
            return f'https://payment.revtel-api.com/{self.VERSION}'

    def _post(self, path, data):
        url = f'{self.host}/{path}'
        return requests.post(url, json=data, headers={'x-api-key': self.API_KEY, 'Content-Type': 'application/json'})

    def build_issue_data(self, **kwargs):
        raise NotImplementedError

    def issue_invoice(self, **kwargs):
        if not self.order:
            data = kwargs
        else:
            data = self.build_issue_data(**kwargs)
        path = f'{self.invoice_type}/invoice/issue'
        resp = self._post(path, data=data)
        resp.raise_for_status()
        result = {
            'req_data': data,
            'resp_data': resp.json()
        }
        invoice = self.on_invoice_result(result)
        return invoice

    def map_result(self, data):
        raise NotImplementedError

    def on_invoice_result(self, result):
        try:
            mapped = self.map_result(result)
        except Exception as e:
            msg = json.dumps(result)
            raise MapResultError(msg)

        if not mapped:
            return None
        return self.invoice_class.objects.create(
            order=self.order,
            carrier_type=self.carrier_type,
            carrier_number=self.carrier_number,
            category=self.category,
            uni_no=self.uni_no,
            amount=self.cart['calculations']['amount'],
            detail=json.dumps(result, ensure_ascii=False),
            invoice_type=self.invoice_type,
            **mapped
        )


class EcpayProvider(BaseProvider):
    invoice_type = 'ecpay'

    def map_result(self, data):
        payload = {}
        result = data['resp_data']
        if result.get('RtnCode') == 1:
            payload['invoice_status'] = 'success'
            no_str = 'InvoiceNumber' if self.category == 'B2B' else 'InvoiceNo'
            payload['invoice_number'] = result[no_str]
        else:
            payload['invoice_status'] = 'failure'

        return payload
            

    def build_items(self):
        items = [
            {
                'ItemSeq': index + 1,
                'ItemName': item['name'],
                'ItemCount': item['quantity'],
                'ItemWord': '個',
                'ItemPrice': item['price'],
                'ItemAmount': item['amount'],
                "ItemTaxType": "1",
            } for index, item in enumerate(self.items)
        ]
        return items

    def build_issue_data(self, **kwargs):
        calc = self.cart['calculations']
        config = self.cart['config']
        extra = {}
        if self.category == 'B2B':
            extra['CustomerIdentifier'] = self.uni_no
            extra['Identifier'] = self.uni_no
            extra['SalesAmount'] = calc['sales']
            extra['TaxAmount'] = calc['tax']
            extra['TotalAmount'] = calc['amount']
            extra['CompanyName'] = self.company_title
            extra['Address'] = config['address']
            extra['EmailAddress'] = config['email']
            extra['TelephoneNumber'] = config['phone']
            extra['ExchangeMode'] = '1'
            extra['type'] = '1'
            extra['Action'] = 'Add'
        else:
            extra['Donation'] = '1' if self.carrier_type == 'donation' else '0'
            if extra['Donation'] == '1':
                extra['LoveCode'] = self.carrier_number
            else:
                types = {
                    'default': '1',
                    'npc': '2',
                    'mobile': '3',
                    'donation': ''
                }
                extra['CarrierType'] = types[self.carrier_type]
                extra['CarrierNum'] = self.carrier_number
            extra['Print'] = "0"
            extra['SalesAmount'] = round(calc['amount'])

        return {
            'category': self.category,
            'TimeStamp': int(timezone.now().timestamp()),
            'RelateNumber': self.invoice_id,
            'CustomerEmail': config['email'],
            'TaxType': '1',  # 課稅類別
            'InvType': '07',
            'Items': self.build_items(),
            'vat': '1',
            **extra,
            **kwargs
        }


MAPPING = {
    'mobile': '0',
    'npc': '1',
    'default': '2'
}


class NewebProvider(BaseProvider):
    invoice_type = 'neweb'

    def map_result(self, data):
        resp_data = data['resp_data']
        result = resp_data['Result']
        result = result if type(result) is dict else json.loads(result)
        payload = {}
        if resp_data['Status'] == 'SUCCESS':
            payload['invoice_status'] = 'success'
            payload['invoice_number'] = result['InvoiceNumber']
        else:
            payload['invoice_status'] = 'failure'
        return payload

    def build_items(self):
        return self.items

    def build_detail_data(self):
        data = {}
        if self.category == 'B2C':
            data = {
                'Category': 'B2C',
                'PrintFlag': "N",
                'CarrierType': MAPPING[self.carrier_type],
                'CarrierNum': self.carrier_number,
            }

        elif self.category == 'B2B':
            data = {
                'Category': 'B2B',
                'PrintFlag': "Y",
                'BuyerUBN': self.uni_no,
                'BuyerName': self.company_title
            }

        if self.carrier_type == 'donation':
            data = {
                'PrintFlag': "N",
                'LoveCode': self.carrier_number
            }

        return data

    def build_general_data(self):
        now = timezone.now()
        config = self.cart['config']
        tax = self.cart['calculations']['tax']
        return {
            'RespondType': "JSON",
            'Version': "1.4",
            'TimeStamp': str(int(now.timestamp())),
            'MerchantOrderNo': self.invoice_id,
            'Status': '1',
            'Category': self.category,
            'BuyerName': config.get('name'),
            'BuyerEmail': self.receiver_email,
            'PrintFlag': "Y",
            'TaxType': '1',
            'TaxRate': float(5),
            'Amt': self.cart['calculations']['sales'],
            'TaxAmt': tax,
            'TotalAmt': self.cart['calculations']['amount'],
        }

    def build_issue_data(self):
        resp_data = {
            **self.build_general_data(),
            **self.build_detail_data(),
        }
        return {
            **resp_data,
            'items': self.build_items()
        }



from django.db import models
from revpayment.settings import api_settings
from django.conf import settings
from django.utils import timezone
from uuid import uuid4
from django.contrib import admin
import json
import random
import string
import uuid


def uuid_hex():
    return uuid.uuid4().hex[:-2]


def random_digits(stringLength=8):
    letters = string.digits
    return ''.join(random.choice(letters) for i in range(stringLength))


def gen_order_id():
    now = timezone.now()
    return now.strftime('%Y%m%d%H%M%S') + random_digits(4)


def empty_cart():
    from revpayment.carts import Cart
    return json.dumps(Cart.get_empty_cart())


class PaymentInfo(models.Model):
    cart = models.TextField(blank=True, default=empty_cart)
    credit = models.IntegerField(default=0)
    trading_slang = models.CharField(max_length=20, default=random_digits)
    owner = models.OneToOneField(api_settings.BUYER_CLASS, on_delete=models.CASCADE, related_name='payment')
    uid = models.CharField(max_length=30, default=uuid_hex)

    class Meta:
        abstract = 'revpayment' not in settings.INSTALLED_APPS
        app_label = 'revpayment'


class RedirectState(models.Model):
    PAYMENT_TYPES = (
        ('offline', 'offline'),
        ('ecpay', 'ecpay'),  # 綠界支付 #
        ('neweb', 'neweb'),  # 藍新支付 #
    )

    PAYMENT_SUBTYPES = (
        ('default', 'default'),  # 預設全開 #
        ('credit', 'credit'),  # 信用卡 #
        ('cvs_cod', 'cvs_cod'),  # 超商貨到付款 #
        ('cvs', 'cvs'),  # 超商付款 #
        ('atm', 'atm'),  # atm #
        ('web_atm', 'web_atm'),  # web_atm #
        ('barcode', 'barcode'),  # 條碼付款 #
    )

    state = models.UUIDField(default=uuid4)
    redirect_url = models.URLField(default=api_settings.WEB_HOST)
    payment_type = models.CharField(
        max_length=20, choices=PAYMENT_TYPES, default='neweb')
    payment_subtype = models.CharField(
        max_length=20, choices=PAYMENT_SUBTYPES, default='default')
    cart = models.TextField()
    order_id = models.CharField(max_length=128, default=gen_order_id)
    order_type = models.CharField(max_length=128, default='default')
    buyer = models.ForeignKey(api_settings.BUYER_CLASS, on_delete=models.CASCADE, null=True)

    class Meta:
        abstract = 'revpayment' not in settings.INSTALLED_APPS
        app_label = 'revpayment'


class BaseOrder(models.Model):
    CURRENCY_CHOICES = (
        ('TWD', 'TWD'),
        ('USD', 'USD'),
        ('JPY', 'JPY'),
    )

    PAYMENT_TYPES = (
        ('offline', 'offline'),
        ('ecpay', 'ecpay'),  # 綠界支付 #
        ('neweb', 'neweb'),  # 藍新支付 #
    )

    PAYMENT_SUBTYPES = (
        ('default', 'default'),  # 預設全開 #
        ('credit', 'credit'),  # 信用卡 #
        ('cvs_cod', 'cvs_cod'),  # 超商貨到付款 #
        ('cvs', 'cvs'),  # 超商付款 #
        ('atm', 'atm'),  # atm #
        ('web_atm', 'web_atm'),  # web_atm #
        ('barcode', 'barcode'),  # 條碼付款 #
    )

    PAYMENT_STATUS = (
        ('success', 'success'),
        ('failure', 'failure'),
        ('refunded', 'refunded'),
        ('logistics_created', 'logistics_created'),
        ('form_created', 'form_created'),
        ('code_generated', 'code_generated'),
    )

    TYPES = (
        ('default', 'default'),
        ('credit', 'credit')
    )

    id = models.CharField(
        max_length=128, primary_key=True, default=gen_order_id)
    buyer = models.ForeignKey(api_settings.BUYER_CLASS, on_delete=models.CASCADE)
    order_type = models.CharField(
        max_length=128, choices=TYPES, default='default')
    amount = models.FloatField(default=0)
    title = models.CharField(max_length=100, default='order')
    description = models.CharField(max_length=256, default='no description')
    currency = models.CharField(
        max_length=8, choices=CURRENCY_CHOICES, default='TWD')
    payment_type = models.CharField(
        max_length=20, choices=PAYMENT_TYPES, default='neweb')
    payment_subtype = models.CharField(
        max_length=20, choices=PAYMENT_SUBTYPES, default='default')
    payment_status = models.CharField(
        max_length=20, choices=PAYMENT_STATUS, default='waiting')
    data = models.TextField(null=True, blank=True)
    cart = models.TextField(null=True, blank=True)
    receiver_email = models.EmailField()
    receiver_name = models.CharField(max_length=20, blank=True)
    receiver_phone = models.CharField(max_length=20, blank=True)
    receiver_address = models.CharField(max_length=128, blank=True)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    payment_transaction_detail = models.TextField(blank=True)
    payment_redirect_detail = models.TextField(blank=True)

    code_no = models.CharField(max_length=128, blank=True)
    bank_code = models.CharField(max_length=128, blank=True)
    bank_account = models.CharField(max_length=128, blank=True)
    barcode_1 = models.CharField(max_length=128, blank=True)
    barcode_2 = models.CharField(max_length=128, blank=True)
    barcode_3 = models.CharField(max_length=128, blank=True)
    pay_deadline = models.DateTimeField(null=True, blank=True)

    class Meta:
        abstract = True

    def __str__(self):
        return self.id


class Order(BaseOrder):
    class Meta:
        abstract = 'revpayment' not in settings.INSTALLED_APPS
        app_label = 'revpayment'


if settings.REVPAYMENT.get('ORDER_CLASS', 'revpayment.models.Order') == 'revpayment.models.Order' and 'revpayment' in settings.INSTALLED_APPS:
    admin.site.register(Order)



from rest_framework import viewsets, response, permissions, exceptions, views
from revpayment.carts import Cart
from rest_framework.decorators import action
from revpayment.settings import api_settings
from revpayment.models import RedirectState
from django.shortcuts import redirect
from django.contrib.auth.models import User
import jwt
import json
from revpayment.checkout import check_certificate

verify_token_method = api_settings.VERIFY_METHOD


class CartViewSet(viewsets.ViewSet):
    permission_classes = [permissions.IsAuthenticated, ]
    add_item_path = 'item/add'
    edit_item_path = 'item/edit'
    delete_item_path = 'item/delete'
    edit_config_path = 'config/edit'
    clear_path = 'clear'
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get_cart(self):
        return self.cart_class(self.request.user.profile)

    def list(self, request):
        cart = self.get_cart()
        return response.Response(cart.cart, 200)

    @action(detail=False, url_path=clear_path, methods=['POST'])
    def clear(self, request):
        cart = self.get_cart()
        cart.clear_cart()
        return response.Response(cart.cart, 200)

    @action(detail=False, url_path=add_item_path, methods=['POST'])
    def add_item(self, request):
        cart = self.get_cart()
        item = self.request.data
        cart.add_item(item)
        return response.Response(cart.cart, 200)

    @action(detail=False, url_path=edit_item_path, methods=['POST'])
    def edit_item(self, request):
        cart = self.get_cart()
        data = self.request.data
        if 'index' not in data:
            raise exceptions.ValidationError(
                code='index_is_required', detail='index is required')
        if not data.get('config'):
            raise exceptions.ValidationError(
                code='config_is_required', detail='config is required')
        cart.edit_item(index=data['index'], config=data['config'])
        return response.Response(cart.cart, 200)

    @action(detail=False, url_path=delete_item_path, methods=['POST'])
    def delete_item(self, request):
        cart = self.get_cart()
        data = self.request.data
        if 'index' not in data:
            raise exceptions.ValidationError(
                code='index_is_required', detail='index is required')
        cart.delete_item(index=data['index'])
        return response.Response(cart.cart, 200)

    @action(detail=False, url_path=edit_config_path, methods=['POST'])
    def edit_config(self, request):
        cart = self.get_cart()
        data = self.request.data
        cart.edit_config(data)
        return response.Response(cart.cart, 200)


class CheckoutView(views.APIView):
    permission_classes = []
    redirect_base = api_settings.WEB_HOST
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get(self, request):
        token = self.request.query_params.get('token')
        token_type = self.request.query_params.get('token_type')
        payment_type = self.request.query_params.get('payment_type', 'neweb')
        payment_subtype = self.request.query_params.get(
            'payment_subtype', 'default')
        if not token:
            raise exceptions.ValidationError(code='token_required')
        if token_type == 'token':
            try:
                user = User.objects.get(auth_token__key=token)
                profile = user.profile
            except:
                raise exceptions.PermissionDenied
        else:
            profile = verify_token_method(token)
        state = RedirectState.objects.create(
            buyer_id=profile.id,
            redirect_url=self.redirect_base,
            payment_type=payment_type,
            payment_subtype=payment_subtype,
            cart=profile.payment.cart
        )
        sdk = self.sdk_class(state=state)
        return sdk.checkout()


class CallbackView(views.APIView):
    permission_classes = []
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def post(self, request):
        data = request.data
        state, data = check_certificate(data['certificate'])
        sdk = self.sdk_class(state=state)
        sdk.callback(data)
        return response.Response('1', 200)


class ClientRedirect(views.APIView):
    permission_classes = ()
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get(self, request):
        cert = request.query_params.get('certificate')
        state, data = check_certificate(cert)
        client = self.sdk_class(state=state)
        url = client.customer_redirect(data)
        return redirect(url)

    def post(self, request):
        state, data = check_certificate(request.data['certificate'])
        client = self.sdk_class(state=state)
        url = client.customer_redirect(data)
        return redirect(url)


class CartView(views.APIView):
    permission_classes = (permissions.IsAuthenticated, )
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get_cart(self):
        return self.cart_class(self.request.user.profile)

    def get(self, request):
        cart = self.get_cart()
        return response.Response(cart.cart, 200)


class ClearCartView(views.APIView):
    permission_classes = (permissions.IsAuthenticated, )
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get_cart(self):
        return self.cart_class(self.request.user.profile)

    def post(self, request):
        cart = self.get_cart()
        cart.clear_cart()
        return response.Response(cart.cart, 200)


class AddItemView(views.APIView):
    permission_classes = (permissions.IsAuthenticated, )
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get_cart(self):
        return self.cart_class(self.request.user.profile)

    def post(self, request):
        cart = self.get_cart()
        cart.add_item(request.data)
        return response.Response(cart.cart, 200)


class EditItemView(views.APIView):
    permission_classes = (permissions.IsAuthenticated, )
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get_cart(self):
        return self.cart_class(self.request.user.profile)

    def post(self, request):
        cart = self.get_cart()
        data = self.request.data
        if 'index' not in data:
            raise exceptions.ValidationError(
                code='index_is_required', detail='index is required')
        if not data.get('config'):
            raise exceptions.ValidationError(
                code='config_is_required', detail='config is required')
        cart.edit_item(index=data['index'], config=data['config'])
        return response.Response(cart.cart, 200)


class DeleteItemView(views.APIView):
    permission_classes = (permissions.IsAuthenticated, )
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get_cart(self):
        return self.cart_class(self.request.user.profile)

    def post(self, request):
        cart = self.get_cart()
        data = self.request.data
        if 'index' not in data:
            raise exceptions.ValidationError(
                code='index_is_required', detail='index is required')
        cart.delete_item(index=data['index'])
        return response.Response(cart.cart, 200)


class EditConfigView(views.APIView):
    permission_classes = (permissions.IsAuthenticated, )
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get_cart(self):
        return self.cart_class(self.request.user.profile)

    def post(self, request):
        cart = self.get_cart()
        data = self.request.data
        cart.edit_config(data)
        return response.Response(cart.cart, 200)


class CreditView(views.APIView):
    permission_classes = ()
    item_class = api_settings.ITEM_CLASS
    calc_class = api_settings.CALC_CLASS
    redirect_base = api_settings.WEB_HOST
    cart_class = api_settings.DEFAULT_CART_CLASS
    sdk_class = api_settings.DEFAULT_SDK_CLASS

    def get(self, request):
        credit = self.request.query_params.get('credit')
        token_type = self.request.query_params.get('token_type')
        token = self.request.query_params.get('token')
        payment_type = self.request.query_params.get('payment_type', 'neweb')
        payment_subtype = self.request.query_params.get('payment_subtype', 'credit')
        if token_type == 'token':
            try:
                user = User.objects.get(auth_token__key=token)
                profile = user.profile
            except:
                raise exceptions.PermissionDenied
        else:
            profile = verify_token_method(token)

        credit = int(credit) if type(credit) is str else credit
        item = self.item_class.get_credit_item(credit)
        cart = self.cart_class(profile)
        config = cart.cart['config']
        calc = self.calc_class(items=[item], profile=profile, config=cart.cart['config'])

        cart_data = {'config': config, 'calculations': calc.get_calc_dict(), 'items': [item]}
        state = RedirectState.objects.create(
            order_type='credit',
            cart=json.dumps(cart_data, ensure_ascii=False),
            buyer=profile,
            payment_type=payment_type,
            payment_subtype=payment_subtype
        )

        sdk = self.sdk_class(state=state)
        return sdk.checkout()



from rest_framework import routers
from .views import CartViewSet, CheckoutView, CallbackView, ClientRedirect, CartView, AddItemView, EditItemView, EditConfigView, DeleteItemView, ClearCartView, CreditView
from revpayment.logistics.views import LogisticCallBack, SelectCVS, CVSCallback
from django.urls import path

router = routers.DefaultRouter(trailing_slash=False)
router.register('cart', CartViewSet, basename='cart')
cart_urls = router.urls

checkout_urlpatterns = [
    path('checkout', CheckoutView.as_view(), name='payment-checkout'),
    path('callback', CallbackView.as_view(), name='payment-callback'),
    path('redirect', ClientRedirect.as_view(), name='payment-redirect'),
    path('credit', CreditView.as_view(), name='payment-credit'),
    path('logistics/callback', LogisticCallBack.as_view(), name='logistic-callback'),
    path('logistics/cvs/select', SelectCVS.as_view(), name='logistic-cvs-select'),
    path('logistics/cvs/callback', CVSCallback.as_view(), name='logistic-cvs-callback'),
]

cart_urlpatterns = [
    path('', CartView.as_view(), name='cart-get'),
    path('clear', ClearCartView.as_view(), name='cart-clear'),
    path('item/add', AddItemView.as_view(), name='cart-add-item'),
    path('item/edit', EditItemView.as_view(), name='cart-edit-item'),
    path('item/delete', DeleteItemView.as_view(), name='cart-delete-item'),
    path('config/edit', EditConfigView.as_view(), name='cart-edit-config'),
]



from revpayment.models import RedirectState
from revpayment.settings import api_settings


class BaseHandler:
    tracking_fields = {
        'payment_status': 'payment'
    }

    def __init__(self, order):
        self.order = order

    def get_state_meta(self, state):
        from revpayment.carts import Cart
        buyer = api_settings.BUYER_CLASS.objects.get(id=state.buyer_id)
        cart = Cart(buyer)
        return cart, buyer

    def on_payment_result(self, changed):
        created = changed.get('__created', False)
        if created:
            self.on_order_create()
        for k, v in changed.items():
            if k in self.tracking_fields:
                previous = changed.get(f'_{k}', None)
                func = getattr(self, f'on_{self.tracking_fields[k]}_{v}', None)
                if func:
                    func(previous)

    def before_checkout(self, state):
        raise NotImplementedError

    def after_checkout(self, state):
        raise NotImplementedError

    def on_order_create(self):
        raise NotImplementedError


class Handler(BaseHandler):
    def on_payment_success(self, previous):
        if self.order.order_type == 'credit':
            buyer = self.order.buyer
            buyer.credit += self.order.amount
            buyer.save()

    def on_payment_failure(self, previous):
        return

    def before_checkout(self, state):
        return

    def after_checkout(self, state):
        return

    def on_order_create(self):
        return



from rest_framework import views, response
from revpayment.settings import api_settings
from revpayment.logistics import LogisticsSDK
from revpayment.models import RedirectState

order_class = api_settings.ORDER_CLASS
verify_method = api_settings.VERIFY_METHOD
# Create your views here.


class LogisticCallBack(views.APIView):
    permission_classes = ()

    def post(self, request):
        data = request.data
        state = self.request.query_params.get('state')
        state = RedirectState.objects.get(state=state)
        order = order_class.objects.get(id=state.order_id)
        sdk = LogisticsSDK(order=order)
        sdk.callback(data)
        return response.Response('1', 200)


class SelectCVS(views.APIView):
    permission_classes = ()

    def get(self, request):
        token = request.query_params.get('token')
        logistic_provider = request.query_params.get('logistic_provider', 'ecpay')
        logistic_subtype = request.query_params.get('logistic_subtype', 'UNIMARTC2C')
        payment_type = request.query_params.get('payment_type', 'ecpay')
        payment_subtype = request.query_params.get('payment_subtype', 'default')
        collection = payment_subtype == 'cvs_cod'
        profile = verify_method(token)
        state = RedirectState.objects.create(payment_type=payment_type, payment_subtype=payment_subtype, buyer=profile)
        sdk = LogisticsSDK(logistic_provider=logistic_provider)
        resp = sdk.cvs_map(logistic_subtype, state, collection)
        return resp


class CVSCallback(views.APIView):
    permission_classes = ()

    def post(self, request):
        state = RedirectState.objects.get(state=request.query_params.get('state'))
        sdk = LogisticsSDK()
        return sdk.cvs_callback(request.data.dict(), state.buyer)



from revpayment.settings import api_settings
from revpayment.models import RedirectState
from django.conf import settings
import requests
import json
import jwt
from jwcrypto.jwt import JWK


class CheckoutSDK:
    payment_classes = (
        ('neweb', api_settings.DEFAULT_NEWEB_CLASS),
        ('ecpay', api_settings.DEFAULT_ECPAY_CLASS),
        ('credit', api_settings.DEFAULT_CREDIT_CLASS),
    )
    handler_class = api_settings.HANDLER_CLASS

    def __init__(self, state):
        self.state = state
        if type(state.cart) is str:
            self.cart = json.loads(state.cart)
        elif type(state.cart) is dict:
            self.cart = state.cart
        else:
            raise TypeError
        self.payment_type = state.payment_type
        self.payment_subtype = state.payment_subtype
        self.buyer = state.buyer

    def get_payment_class(self):
        try:
            payment_class = [
                c[1] for c in self.payment_classes if c[0] == self.payment_type][0]
            return payment_class
        except IndexError:
            raise ValueError('payment_type is not valid')

    def get_payment(self):
        payment_class = self.get_payment_class()
        return payment_class(buyer=self.buyer, payment_subtype=self.payment_subtype, cart=self.cart, order_id=self.state.order_id)

    def before_checkout(self):
        handler = self.handler_class(order=None)
        handler.before_checkout(state=self.state)

    def after_checkout(self):
        handler = self.handler_class(order=None)
        handler.after_checkout(state=self.state)

    def checkout(self):
        state = self.state
        self.before_checkout()
        payment = self.get_payment()
        result = payment.checkout(state)
        self.after_checkout()
        return result

    def callback(self, data):
        payment = self.get_payment()
        order, changed = payment.callback(self.state, data)
        handler = self.handler_class(order=order)
        handler.on_payment_result(changed)
        return f'{self.state.redirect_url}/order?id={order.id}'

    def customer_redirect(self, data):
        payment = self.get_payment()
        order, changed = payment.customer_redirect(self.state, data)
        handler = self.handler_class(order=order)
        handler.on_payment_result(changed)
        return f'{self.state.redirect_url}/order?id={order.id}'


class BaseCertificate:
    default_aud = settings.CLIENT_ID

    def __init__(self, cert):
        self.unverified = jwt.decode(cert, verify=False)
        self.headers = jwt.get_unverified_header(cert)
        self.cert = cert

    def get_order_id(self, sub):
        raise NotImplementedError

    def get_key_url(self):
        iss = self.unverified['iss']
        host = iss.split('.')[0]
        if host[-4:] in ['-stg', '-dev']:
            return 'https://auth-stg.revtel-api.com/v3/certs'
        return 'https://auth.revtel-api.com/v3/certs'

    def get_pub_key(self):
        url = self.get_key_url()
        jwks = requests.get(url).json()['keys']
        key = [jwk for jwk in jwks if jwk['kid'] == self.headers['kid']][0]
        real_key = JWK.from_json(json.dumps(key))
        return real_key.export_to_pem()

    def verify(self):
        pub = self.get_pub_key()
        decoded = jwt.decode(self.cert, algorithms='RS256', key=pub, verify=True, audience=self.default_aud)
        if decoded['typ'] != 'certificate':
            raise

        sub = decoded['sub']
        order_id = self.get_order_id(sub)
        state = RedirectState.objects.get(order_id=order_id)
        return state, sub


class NewebCertificate(BaseCertificate):
    def get_order_id(self, sub):
        return sub['Result']['MerchantOrderNo']


class EcpayCertificate(BaseCertificate):
    def get_order_id(self, sub):
        return sub['MerchantTradeNo']


def check_certificate(cert):
    unverified = jwt.decode(cert, verify=False)
    provider = unverified['provider']
    if provider == 'ecpay':
        cert_class = EcpayCertificate
    elif provider == 'neweb':
        cert_class = NewebCertificate

    certificate = cert_class(cert)
    return certificate.verify()
